/****************************************************************************
Date: 9/9/2001
File: cfg.cpp
Description:	CONFIGURATION SERVER CLASS
History:
	4/26/98 Added support for select file (multiple modes)
	4/6/99  Added default configuration commands
	5/16/99 "downlink" keywords
	6/5/00  Added ICOM ic821 radio type
	7/20/00 Added FT100 
	3/26/01 Added FT817 G6LVB
	6/14/2001 Added TS-790 
	8/18/2001 parameter name change in radio.hpp (unParam1)
	9/2/2001 unParam1 is HEX or decimal depending on radio type
	9/9/2001 added TS-2000
*****************************************************************************/
#include <stddef.h>
#include <string.h>

#include "cfg.hpp"

//***** Radio lookup table ****************
// must be in same order as RADIOTYPE enum in RADIO.HPP
const CHAR* CONFIG::aRadioNames[] = {
	"null",
	"kenwood",
	"micbutton",
	"ft736",
	"ft847",
	"ft100",                 
	"ft817", // G6LVB Mar 26 2001
	"ic821",
	"ts790",
	"ts2000"
};

//***** Modulation Lookup Table ************
// must be in same order as MODULATION enum in MODULATE.HPP
const CHAR* CONFIG::aModulationNames[] = {
	"",	// N/A
	"lsb",
	"usb",
	"cw",
	"fm",
	"am",
	"fsk"
};
  
//***** Keyword table must be in same order as KEYWORD enum *****
const CHAR* CONFIG::aKeyWords[] = {
	"",
	"default",
	"satellite",
	"transponder",
	"transponder2",
	"beacon",
	"downlink",
	"downlink2",
	"uplink",
	"rx",
	"rx2",
	"tx",
	"xcvr",
	"rxconverter",
	"rxconverter2",
	"txconverter",
	"comport"
};
const CHAR* const CONFIG::INVERT = "invert";	// special keyword not in table
						// for inverting transponder

// Constructor
CONFIG::CONFIG() :

	pConfig(&DefaultConfig),
	RadioTable(CONFIG::aRadioNames, sizeof(CONFIG::aRadioNames)),
        ModulationTable(aModulationNames, sizeof(aModulationNames)),
        KeyWordTable(aKeyWords, sizeof(aKeyWords)),
	Parser(),
	eError(BOOL_FALSE) {

	memset(szName, 0 , sizeof(szName));
	memset(szConfigID, 0 , sizeof(szConfigID));
	memset(&DefaultConfig, 0, sizeof(DefaultConfig));
	memset(aComPortInfo, 0, sizeof(aComPortInfo));
}

// Functions for keyword processing
VOID	CONFIG::SetTranslation(TRANSPONDERPARAMS& Params) {

        Params.dParam = GetFreq();
        if ( 0 == strcmp(Parser.GetString(), INVERT) ) {
		Params.eType = TRANSPONDERTYPE_INVERTING;
	}
	else {
		Params.eType = TRANSPONDERTYPE_NONINVERTING;
	}
	return;
}

VOID	CONFIG::SetLink(DOUBLE& dFrequency, MODULATION& eMode) {

	dFrequency = GetFreq();
	eMode = GetModulation(Parser.GetString());
	return;
}

VOID	CONFIG::SetRadio(RADIOPARAMS& Params) {

	//***** Get Radio Type using Radio Name *******************
	CHAR* pszRadioName = Parser.GetString();
        Params.eType = GetRadioType(pszRadioName);
        if (Params.eType == RADIOTYPE_NULL) {
		Error(CONFIG_ERROR_UNKNOWNRADIO);
		return;
	}

        //**** Build up configuration record ***********************
	Params.unPort = (USHORT) Parser.GetInteger(); // Get COM or LPT port
	if (Params.eType == RADIOTYPE_MICBUTTON) {
                //***** Parallel Port (mic button) radios *************
                // Get mic button radio parameters
                Params.ulInitialFreq = (ULONG) GetFreq();
                Params.ulFreqStep    = (ULONG) Parser.GetInteger();
	}
        else {
                //***** Serial port radios *****************************
		// Check COM port number
		if ( (Params.unPort == 0) || (Params.unPort > MAXCOMPORT) ) {
			Error(CONFIG_ERROR_COMPORT);
			return;
		}

                // Get serial port parameters
                UINT unIndex   = Params.unPort - 1;
                Params.unIRQ   = aComPortInfo[unIndex].unIRQ;
                Params.Address = aComPortInfo[unIndex].Address;
                Params.unRate  = aComPortInfo[unIndex].unRate;
		if (Params.eType == RADIOTYPE_ICOM) {
			// This is the HEX ICOM address
			Params.unParam1 = (USHORT) Parser.GetHex();
		}
		else {
			// This is a radio-specific param if used
			Params.unParam1 = (USHORT) Parser.GetInteger();
		}
	}
	
	return;
}

VOID	CONFIG::SetConverter(DOUBLE& dConverter) {
        dConverter = GetFreq();
	return;
}

BOOL	CONFIG::ProcessLine() {

        USHORT	unPort;
	BOOL	eDone 	 = BOOL_FALSE;

	const CHAR* pszToken = Parser.GetToken();

	if (pszToken == NULL) {
		return BOOL_TRUE;	// EOF so done!
	}

	switch( GetKeyWord(pszToken) ) {

	case KEYWORD_NOTFOUND:
		Error(CONFIG_ERROR_NOTFOUND);
		break;

	case KEYWORD_DEFAULT:
		// This lets user set a default config.
		// Set config pointer for default config
		pConfig = &DefaultConfig;
		break;

	case KEYWORD_SATELLITE:
		// save name and mode of next sat
		strncpy(szName, Parser.GetString(), SATNAMESIZE);
		strncpy(szConfigID, Parser.GetString(), CONFIGIDSIZE);

		// Make sure name and mode is null terminated
		szName[SATNAMESIZE] = 0;
		szConfigID[CONFIGIDSIZE] = 0;

		// set config pointer for normal config record
		pConfig = &Config;

		// end of this config record
        	eDone = BOOL_TRUE;
		break;

	case KEYWORD_TRANSLATION1:
        	SetTranslation(pConfig->SatParams.Main);
		break;

	case KEYWORD_TRANSLATION2:
        	SetTranslation(pConfig->SatParams.Aux);
		break;

	case KEYWORD_BEACON:
	case KEYWORD_DOWNLINK:
		SetLink(pConfig->SatParams.Main.dOutFreq,
			pConfig->SatParams.Main.eOutMode );
		break;

	case KEYWORD_DOWNLINK2:
		SetLink(pConfig->SatParams.Aux.dOutFreq,
			pConfig->SatParams.Aux.eOutMode );
		break;

	case KEYWORD_UPLINK:
		pConfig->SatParams.Main.eType = TRANSPONDERTYPE_FIXED;
		SetLink(pConfig->SatParams.Main.dParam,
			pConfig->SatParams.Main.eInMode);
		break;

	case KEYWORD_RX:
        	SetRadio(pConfig->RxParams);
		break;

	case KEYWORD_RX2:
        	SetRadio(pConfig->Rx2Params);
		break;

	case KEYWORD_TX:
        	SetRadio(pConfig->TxParams);
		break;

	case KEYWORD_XCVR:
		SetRadio(pConfig->RxParams);
		pConfig->RxParams.eTransceive = BOOL_TRUE;
		break;

	case KEYWORD_RXCONVERTER:
        	SetConverter(pConfig->RxParams.dRxConverter);
		break;

	case KEYWORD_RXCONVERTER2:
        	SetConverter(pConfig->Rx2Params.dRxConverter);
		break;

	case KEYWORD_TXCONVERTER:
        	SetConverter(pConfig->TxParams.dTxConverter);
		break;

	case KEYWORD_COMPORT:
		unPort = (USHORT) Parser.GetInteger();
        	if ( (unPort == 0) || (unPort > MAXCOMPORT) ) {
			Error(CONFIG_ERROR_COMPORT);
		}
		else {
			// Set Com Port Params
			--unPort;	// use port number as an index
			aComPortInfo[unPort].unRate = (USHORT)
							Parser.GetInteger();
			aComPortInfo[unPort].unIRQ = (USHORT)
							Parser.GetInteger();
			aComPortInfo[unPort].Address = (IOADDRESS)
							 Parser.GetHex();
		}
		break;

	default:
		Error(CONFIG_ERROR_SOFTWARE);
		break;
	} // end switch()

	return eDone;
}
 
// G6LVB added start...
#if _MSC_VER==800                    
	#pragma optimize("lge",off)
#endif
// ...G6LVB added end

// GetFreq() utility
DOUBLE	CONFIG::GetFreq() {
	return Parser.GetFloat() * 1.0e6;
}   

// G6LVB added start...
#if _MSC_VER==800                    
	#pragma optimize("lge",on)
#endif
// ...G6LVB added end

BOOL	CONFIG::Find(const CHAR FAR* lpszSatName, const CHAR* pszDesired) {

	while( Read() ) {

		// Match Satellite name and desired config ID 
		if ( (!_fstricmp(GetSatelliteName(), lpszSatName)) && 
		     ( (*pszDesired == CONFIG_FIRST) || 
		       (!stricmp(GetConfigID(), pszDesired))
		     )
		   ) {

			return BOOL_TRUE;
		}
	}
	return BOOL_FALSE;
}

BOOL	CONFIG::Read() {

	if ( Parser.IsEOF() ) {
		return BOOL_FALSE;
	}

	// start with default configuration
	memcpy(&Config, &DefaultConfig, sizeof(Config));

	while( !ProcessLine() ) {
		if ( AnyErrors() )  {
			return BOOL_FALSE;
		}
	}
	return BOOL_TRUE;
}

CONFIGINFO* CONFIG::GetConfig(const CHAR FAR* lpszSatName) {

	// Get id of desired config for this satellite
	// default is FIRST one if no SELECTFILE or no entry for this sat
	CHAR	szDesiredID[sizeof(szConfigID)];
	szDesiredID[0] = CONFIG_FIRST ;

	if ( Parser.Open(CONFIG_SELECTFILE) ) {
		while( !Parser.IsEOF() ) {
			if ( !_fstricmp(lpszSatName, Parser.GetToken()) ) {
				strncpy(szDesiredID,
					Parser.GetString(), 
					sizeof(szDesiredID));
				break;
			}
		}
		Parser.Close();
	}

	// Now get desired config from config file
	if ( !Parser.Open(CONFIG_FILE) ) {
		Error(CONFIG_ERROR_CFGFILEOPEN);
		return NULL;	// can't open config file
	}

	CONFIGINFO* pConfig = NULL;

	if ( Find(lpszSatName, szDesiredID) ) {

		if ( Read() ) {
			pConfig = &Config;
		}
	}

	Parser.Close();

	return pConfig;
}
// end cfg.cpp
