/**************************************************************************
date: 7/18/2001
file: comdrv.hpp

title:          COMDRV - PC COM Port Driver Class    
  
public:
	static COMDRV* GetThisPointer(UINT unIndex) const

        COMDRV()
        VOID IntHandler()
	BOOL	Open( 	UINT	unComPort,
			UINT	unBitRate,
                        UINT    unDataBits,
                        UARTPARITY eParity,
                        UINT    unStopBits,
			UINT	unIRQ 		= DEFAULTIRQ,
			IOADDRESS Address 	= DEFAULTADDRESS)
        VOID    Putc(CHAR cChar)
	INT	Getc() 
        VOID    Write(const VOID* pUserData, UINT unCount)
	INT	Read(VOID* pUserBuffer, UINT unMaxCount)
	VOID	ClearBuffers()
	VOID	Close()
	~COMDRV()

description:
        Basic buffered COM port driver. Buffer sizes are defined in this
        file. This driver supports 8250 uarts only. This driver responds
	to the CTS lead, if CTS goes low, it will stop transmitting until
	CTS goes high again. It does no other flow control.

	No dynamic memory allocation is used for ease of use in TSR
	environment. Max number of open ports (currently four) is defined
	in comdrv.cpp. If COM 1 to 4 is used, the address and IRQ do not
	need to be specified in Open(). However, Any COM port number,
	at any address with any IRQ line CAN be used.

history: 12/12/96 initial version
	 8/14/99 InstantTune 1.07
	 7/18/2001 increase FIFO sizes to 256 bytes
**************************************************************************/
#ifndef COMDRV_HPP 
#define COMDRV_HPP

#pragma check_stack( off )
#pragma check_pointer( off )

#include <stddef.h>
#include <stdlib.h>
#include <dos.h>

#include "loctypes.h"
#include "dospc.h"	// non-standard DOS functions
#include "fastfifo.hpp" // FIFO
#include "uartdrv.hpp"  // UART driver class
#include "picdrv.hpp"	// Programmable Interrupt Controller driver


//***** Constants ************************************
const INT  COMDRV_EOF = FASTFIFO_EOF; // return from Getc() if end of file
const UINT COMDRV_MAXPORTS = 4;	      // max number of com ports open at a time

/******************************************************
 Use these values for the UARTPARITY parameter (defined in uartbase.hpp)

        UARTPARITY_NONE
        UARTPARITY_ODD
        UARTPARITY_MARK
        UARTPARITY_EVEN
        UARTPARITY_SPACE

********************************************************/


class COMDRV {
private:
        //***** Constants **********************************
        enum {
		TXFIFOSIZE = 256,      	// Transmit buffer size (bytes)
		RXFIFOSIZE = 256,      	// Receive buffer size (bytes)
                DEFAULTIRQ = 0,   	// use default IRQ line
                DEFAULTADDRESS   = 0,   // use default uart address
                BIOSSEGMENT = 0x0040    // com port addresses in BIOS
        };

	//***** Flags ****************************************
	BOOL	eOpen;
	BOOL	eTxBusy;

        //***** HW Drivers ***********************************
        UARTDRV 	Uart;	// UART driver
	PICDRV		Pic;	// Programmable Interrupt Controller driver


        //***** FIFO's *************************************
        FASTFIFO        TxFifo;
        FASTFIFO        RxFifo;
	UCHAR		aTxBuffer[TXFIFOSIZE];	// buffer for tx fifo
	UCHAR		aRxBuffer[RXFIFOSIZE];	// buffer for rx fifo

        //***** Data for Interrupt Service Routines ********
        UINT            unIndex;        // index into static ISR data arrays
        static const INTHANDLER aVector[COMDRV_MAXPORTS]; // ISR vectors
	static COMDRV* aThisPointer[COMDRV_MAXPORTS];	// This pointers
	

public:
	static COMDRV* GetThisPointer(UINT unIndex) {
		return aThisPointer[unIndex];
	}
		
        COMDRV();
        VOID IntHandler(); 			// interrupt handler

	BOOL	Open( 	UINT	unComPort,
			UINT	unBitRate,
                        UINT    unDataBits,
                        UARTPARITY eParity,
                        UINT    unStopBits,
			UINT	unIRQ 		= DEFAULTIRQ,
                        IOADDRESS Address       = DEFAULTADDRESS);
        VOID    Putc(CHAR cChar);
        INT     Getc(); 
        VOID    Write(const VOID* pUserData, UINT unCount);
        INT     Read(VOID* pUserBuffer, UINT unMaxCount);
        VOID    ClearBuffers();
        VOID    Close();
        ~COMDRV();
};
#endif // comdrv.hpp
