/****************************************************************************
Date: 3/26/01
File: ft817.hpp
Description:
	Yaseu FT-817 radio driver G6LVB

History:
	A version
*****************************************************************************/
#ifndef FT817_HPP
#define FT817_HPP

#include "loctypes.h"
#include "radio.hpp"	// RADIO base class
#include "comdrv.hpp"	// COM port driver class
#include "inttimer.hpp"	// interval timer


//***** Frequency Steps ****************
const LONG FT817_RXSTEPSIZE	= 10;	// 10 Hz receiver tuning step size
const LONG FT817_TXSTEPSIZE	= 100;	// 100 Hz transmitter tuning step size

//***** Message Definitions ************
// All messages to radio are 5 bytes
const INT FT817_TOMESSAGESIZE	= 5;  	// number of bytes in messages to radio
const INT FT817_FROMMESSAGESIZE	= 5;  	// max bytes in messages from radio

// Received message sizes
const INT FT817_SPLITMODE_OFFSIZE= 1;    // "split mode off" returns 1 byte
const INT FT817_SPLITMODE_ONSIZE= 1;    // "split mode on" returns 1 byte
const INT FT817_VFOMODESIZE= 1;    // "vfo mode" returns 1 byte
const INT FT817_SETFREQSIZE= 1;    // "set frequency" returns 1 byte
const INT FT817_SETMODESIZE= 1;    // "set mode" returns 1 byte
const INT FT817_READTXSTATUSSIZE= 1;	// "read TX status" returns 1 byte
const INT FT817_READFREQMODESIZE= 5;	// "read frequency & mode status" returns 5 bytes

// Sent Message consists of 4 bytes for
// parameters and 1 byte for opcode.
// Opcode is always last.
const INT FT817_MESSAGEPARAMS	= 4;  	// number of params in message to radio
const INT FT817_MESSAGEP1   = 0;    // Param 1 command code goes here
const INT FT817_MESSAGEP4   = 3;    // Param 1 command code goes here

//****** OpCodes ***********************
// Radio command codes
const UCHAR FT817_SPLITMODE_OFF	= 0x82;	// split off
const UCHAR FT817_SPLITMODE_ON	= 0x02;	// split on
const UCHAR FT817_VFOMODE	= 0x81;	// A or B
const UCHAR FT817_SETFREQ	= 0x01;	// set vfo frequency
const UCHAR FT817_SETMODE	= 0x07; // SSB, CW, etc
const UCHAR FT817_READTXSTATUS	= 0xF7;	// read TX status (PTT)
const UCHAR FT817_READFREQMODE  = 0x03; // read frequency & mode

// Mike switch return codes
const UCHAR FT817_NOPTT		= 0xFF; // Manual wrong: TX Status returns FF if in RX


//***** COM PORT PARAMETERS ******************************************
const USHORT FT817_TIMEOUT	= 1;	// 1 second response timer
const UINT	FT817_BITRATE	= 4800; // default bit rate
const UINT	FT817_DATABITS	= 8;
const UINT	FT817_STOPBITS	= 2;
const UARTPARITY FT817_PARITY	= UARTPARITY_NONE;

class FT817 : public RADIO {
private:

	//***** Update State **************************************
	// The "Update" routine is a state machine that processes
	// messages from the radio. The state is needed to handle 
	// received messages properly because responses from the
    // FT-817 do not have an opcode.
	enum STATE {
		STATE_STARTPOLL,	// Start info poll cycle
		STATE_READTXSTATUS,	// Read RX Status (get PTT status)
		STATE_READFREQMODE,	// Status update (VFO freq and mode)
		STATE_CHECK,		// Verify PTT  (uses "Read RX Status")
        STATE_WAITACK,      // Waiting to discard ACK/NACK's from FT-817
        STATE_SETTXFREQ1,
        STATE_SETTXFREQ2,
        STATE_SETTXMODE1,
        STATE_SETTXMODE2,
		STATE_IDLE		// nothing to do
	};

	INTERVALTIMER	Timer;		// used for COM timeout
	STATE		eState; 	// expected message type from radio
	BOOL		eSplitMode;	// G6LVB 13 APR 2001 only split in XCVR mode
	UINT		unMessageSize;	// expected message size from radio

	BOOL		ePTT;		// Push-to-talk status

	// Rx Params
	LONG		lRxVFOSteps;	// last receiver VFO steps sent to radio
	LONG		lRxVFORead;	// freq read from Rx VFO;
	UCHAR		ucRxBand;	// Frequency band read from Rx VFO
	MODULATION	eRxMode;	// current rx modulation type
	UCHAR		ucRxModeRead;	// rx modulation byte from radio
	BOOL		eUserHasTunedRx;// TRUE if user changed VFO freq
	

	// Tx Params
	LONG		lTxVFOSteps;	// last VFO steps sent to radio
	MODULATION	eTxMode;	// current Tx modulation type
	BOOL		eTxLockout;	// lock out TX updates
	
	// Com port driver
	COMDRV		Comport;
	
	// Message Buffers
	struct {
		UCHAR	aParams[FT817_MESSAGEPARAMS];
		UCHAR	ucOpcode;
	} OutputBuffer;

	UCHAR	aInputBuffer[FT817_FROMMESSAGESIZE];
	UINT	unByteCounter;	// received bytes in input buffer
	

	//******************************************************
	// Utilities
	//******************************************************
	
    // Table to convert RADIO::MODULATION type to FT-817 message param
	static const UCHAR aSetModeTable[];

    // Table to convert FT-817 message param to RADIO::MODULATION type
	static const MODULATION aGetModeTable[];
	
	// Send a message to the radio
	VOID	SendMessage(UCHAR ucOpcode) {
		OutputBuffer.ucOpcode = ucOpcode;
		Comport.Write(&OutputBuffer, FT817_TOMESSAGESIZE);
		Timer.Start(1);
	}

	// Send a message with a P1 SubCmd
	VOID	SendMessage(UCHAR ucOpcode, UCHAR ucSubCmd) {
		OutputBuffer.aParams[FT817_MESSAGEP1] = ucSubCmd;
		SendMessage(ucOpcode);
	}

	// Set radio mode
	VOID	SetMode(MODULATION eMode) {
		SendMessage(FT817_SETMODE, aSetModeTable[eMode]);
        eState=STATE_WAITACK;
        unMessageSize=FT817_SETMODESIZE;
	}

	// Set Radio VFO frequency using number of frequency steps
	VOID	SetFrequency(LONG lFreqSteps, LONG lStepSize);

	// Convert Frequency to steps
	LONG	FreqtoSteps(DOUBLE dFrequency, LONG lStepSize) {

		//************************************************
		// WARNING: this code has a workaround for MS C7 bug
		// converting double to long. Do NOT change!
		//************************************************
		volatile ULONG ulFrequency = (ULONG) dFrequency;
		LONG lFrequency = (LONG) ulFrequency;
		return ( (lFrequency) + (lStepSize/2) ) / lStepSize;
	}

	// No copy constructor or assignment allowed
	// Force compile error if attempted
	FT817(const FT817&);
	FT817& operator=(const FT817&);

public:
	// General Functions
	FT817(const RADIOPARAMS& Params);
	BOOL	IsReady();
	VOID	Update();
	~FT817();

	// Tx Functions
	VOID	SetTxFreq(DOUBLE dFrequency);
	VOID	SetTxMode(MODULATION eMode);

	// Rx Functions
	BOOL	HasUserTunedRx() { return eUserHasTunedRx; }
	VOID	RefreshRxParams();
	DOUBLE	GetRxFreq();
	MODULATION GetRxMode() { return eRxMode; }
	VOID	SetRxFreq(DOUBLE dFrequency);
	VOID	SetRxMode(MODULATION eMode);

};
#endif // end FT817.hpp
