/****************************************************************************
Date: 6/3/2000
File: icom.hpp	

Description:
	ICOM radio driver for IC-821
History:
*****************************************************************************/
#ifndef ICOM_HPP
#define ICOM_HPP

#include "loctypes.h"	// local types
#include "radio.hpp"	// RADIO base class
#include "icommsg.hpp"	// ICOM serial port message handler

//***** Frequency Step Sizes ***********************************************
const LONG ICOM_STEPSIZE	= 10;	// 10 Hz frequency tuning step size
const LONG ICOM_FMSTEPSIZE	= 5000;	// 5Khz frequency step in FM mode

class ICOM : public RADIO {
private:
	ICOMMSG		Msg;		// Message handler

	BOOL		eReady;		// ready for next command

	BOOL		eFullDuplex;	// true if full duplex mode
	UCHAR		ucCurrentBand;	// For full Duplex only use
					// ICOMMSG_SUB or ICOMMSG_MAIN

	// Message Processing States
	enum	ICOM_STATE {
		ICOM_STATE_SETTXFREQ,
		ICOM_STATE_SETTXMODE,
		ICOM_STATE_SETRXFREQ,
		ICOM_STATE_SETRXMODE,
		ICOM_STATE_GETRXPARAMS,
		ICOM_STATE_WAITOK,
		ICOM_STATE_IDLE
	};
	ICOM_STATE	eState;		// Message processing state

	// Rx Params
	LONG		lRxVFOSteps;	// receiver VFO steps
	MODULATION	eRxMode;	// current rx modulation type
	BOOL		eUserHasTunedRx;// TRUE if user changed VFO

	// Tx Params
	LONG		lTxVFOSteps;	// last VFO steps sent to radio
	MODULATION	eTxMode;	// current Tx modulation type
	

	//******************************************************
	// Utilities
	//******************************************************

	// Convert DOUBLE Frequency to LONG
	LONG	FreqtoLong(DOUBLE dFrequency) {

		//************************************************
		// WARNING: this code has a workaround for MS C7 bug
		// converting double to long. Do NOT change!
		//************************************************
		volatile ULONG ulFrequency = (ULONG) dFrequency;
		LONG lFrequency = (LONG) ulFrequency;
		return lFrequency;
	}

	// Get frequency step size for this mode
	LONG	GetStepSize(MODULATION eMode) {
		return (eMode == MODULATION_FM) ?
			ICOM_FMSTEPSIZE : ICOM_STEPSIZE;
	}

	// Convert Frequency and step size to number of Steps
	LONG	GetSteps(LONG lFrequency, MODULATION eMode) {
		LONG lStepSize = GetStepSize(eMode);
		return ( lFrequency + (lStepSize/2) ) / lStepSize;
	}


	// Check MAIN or SUB band, change if not the desired band
	BOOL	IsCurrentBand(UCHAR ucDesiredBand) {
		if (!eFullDuplex) {
			ucDesiredBand = ICOMMSG_MAIN;
		}
		if (ucCurrentBand != ucDesiredBand) {
			ucCurrentBand = ucDesiredBand;
			Msg.SendCmd(ICOMMSG_VFOSELECT, ucDesiredBand);
			return BOOL_FALSE;
		}
		return BOOL_TRUE;	// already desired band
	}

	// No copy constructor or assignment allowed
	// Force compile error if attempted
	ICOM(const ICOM&);
	ICOM& operator=(const ICOM&);

public:
	// General Functions
	ICOM(const RADIOPARAMS& Params);
	BOOL	IsReady() { return eReady; }
	VOID	Update();
	~ICOM();

	// Tx Functions
	VOID	SetTxFreq(DOUBLE dFrequency);
	VOID	SetTxMode(MODULATION eMode);

	// Rx Functions
	BOOL	HasUserTunedRx() { return eUserHasTunedRx; }
	VOID	RefreshRxParams();
	DOUBLE	GetRxFreq();
	MODULATION GetRxMode() { return eRxMode; }
	VOID	SetRxFreq(DOUBLE dFrequency);
	VOID	SetRxMode(MODULATION eMode);

};
#endif // end icom.hpp
